PolarBear = {}

function PolarBear:Initialize(obj)
	self.Object = obj
	
	self.Width = 1
	self.Height = 1
	
	self.DrawOrder = DrawOrder.Normal
	
	self.AlwaysDraw = false
	self.CanMove = false
	self.IsInvincible = false
	self.IsDead = false
	self.IsSolid = true
	self.HasSpawned = false
	
	self.Name = "Polar Bear"
	
	self.Armor = Armor.Create()
	self.Armor.Stab = 1.25
	self.Armor.Crush = 0.5
	self.Armor.Slash = 1.5
	self.Armor.Fire = 2.0
	
	self.Skills =
	{
		HP = Skill.Create(),
		Attack = Skill.Create(),
		Defence = Skill.Create()
	}
	
	self.Skills.HP.Xp = self.Skills.HP:XpForLevel(math.random(8, 9))
	self.Skills.Attack.Xp = self.Skills.Attack:XpForLevel(math.random(5, 6))
	self.Skills.Defence.Xp = self.Skills.Defence:XpForLevel(math.random(2, 4))
	
	self.Positions =
	{
		[Direction.Up] = Point.Create(0, -1),
		[Direction.Down] = Point.Create(0, 1),
		[Direction.Left] = Point.Create(-1, 0),
		[Direction.Right] = Point.Create(1, 0)
	}
	
	self.Direction = Direction.Up
	
	self.PathFinder = AStar.Create(self.Object.Stage.Map, self.Object.Objector, 10, 16)
	self.MoveTimer = Timer.Create(0.5, function()
		local p = self.Object.Objector:Get("D132")
		
		if p and not Rectangle.Create(p.Location.X - 1, p.Location.Y - 1, p.Width + 2, p.Height + 2):Collision(Rectangle.Create(self.Location.X, self.Location.Y, self.Width, self.Height)) then
			self.PathFinder:Update(self.Location.X, self.Location.Y, p.Location.X, p.Location.Y)
			self.Location = self.PathFinder.Target
		end
	end)
	
	self.AttackTimer = Timer.Create(1.5, function()
		local p = self.Object.Objector:Get("D132")
		
		if p then
			for key, value in pairs(self.Positions) do
				if p.Location == self.Location + value then
					self:OnAttack({ Plus = value })
				end
			end
		end
	end)
	
	self.Attack = { Type = "Slash", Power = 15 }
	
	self.Font = Font.Create(Game.AbsolutePath .. "Content/Fonts/Font-large.pcx")
end

function PolarBear:OnSpawn(arg)
	self.Location = Point.Create(arg.X, arg.Y)
	self.HasSpawned = true
end

function PolarBear:OnAttack(arg)
	local objs = self.Object.Objector:GetObjectsAtXY(arg.Plus + self.Location)
	local o
	
	for key, value in pairs(objs) do
		if value.OnHit then
			o = value
			break
		end
	end
	
	if not o then
		o = self.Object.Objector:GetObjectAtXY(arg.Plus + self.Location)
	end
	
	if o and not o.IsDead then
		if o.Name then
			Messenger:Add("The " .. self.Name .. " swiped at " .. o.Name .. "!")
			
			if o.OnHit then
				o:OnHit({ Attack = self.Attack, Level = self.Skills.Attack:CurrentLevel() })
			end
		else
			Messenger:Add("The " .. self.Name .. " swiped at nothing...")
		end
	end
end

function PolarBear:OnHit(arg)
	self.Skills.HP.AdditionalXp = self.Skills.HP.AdditionalXp - self.Armor:CalculateDamage(arg.Attack, arg.Level, self.Skills.Defence:CurrentLevel())
	
	if self.Skills.HP:CurrentLevel() < 1 then
		self.IsDead = true
	end
end

function PolarBear:Update()
	self.MoveTimer:Update()
	self.AttackTimer:Update()
	
	if self.IsDead then
		self.Object.Objector:Get("DeathParty"):OnDeathSpawn({ X = self.Location.X, Y = self.Location.Y })
		Messenger:Add("RRRRRAAAAARRRRGGGGHHHH!! *yelp*")
		self.Object:Remove()
	end
end

function PolarBear:Draw(buffer, camera)
	GameObject.Drawing.DrawRectangle(buffer, self, camera, Graphics.MakeColor(0, 0, 0), Graphics.MakeColor(250, 250, 250), "Pb")
end
